# Arcanea Ruby 2D Engine (ARE)
#
# Author:: Josef Smolka (mailto:maurel@gmail.com)
# Copyright:: Copyright (c) 2007 Josef Smolka
# Licence:: MIT Licence (http://www.opensource.org/licenses/mit-license.php)

require 'arevector.rb'

class ARESprite

  public

  # Prepare sprite for usage
  def initialize(parentSurface, hwSurface = false)
    @parentSurface = parentSurface
    # choose surface type
    if hwSurface then
      @flags = SDL::HWSURFACE
    else
      @flags = SDL::SWSURFACE
    end

    @frames = Array.new
    @displayFrame = 0
  end

  # Load sprite from source bitmap
  def sprLoad(sourcePic, frameWidth, framesCount, beginFrame, transparent = true)
    # load base sprite map
    spriteMap = SDL::Surface.load(sourcePic)
    # load frames
    framesCount.times do |fi|
      newFrame = SDL::Surface.new(SDL::HWSURFACE, frameWidth, spriteMap.h, @parentSurface)
      SDL.blitSurface(spriteMap, fi * frameWidth, 0, frameWidth, spriteMap.h, newFrame, 0, 0)
      newFrame.setColorKey(SDL::SRCCOLORKEY, newFrame[0, 0]) if transparent
      @frames << newFrame
    end

    @displayFrame = beginFrame
  end

  # Set transparency for specific frame
  def sprSetTransparencyFrame(srcX, srcY, frameId)
    if frameId.abs >= @frames.size then
      raise IndexError, "Sprite frame ERROR: #{frameId} >= size (#{@frames.size})"
    end
    @frames[frameId.abs].setColorKey(SDL::SRCCOLORKEY, newFrame[srcX, srcY])
  end

  # Set transparency for all frames
  def sprSetTransparency(srcX, srcY)
    @frames.each do |frame|
      frame.setColorKey(SDL::SRCCOLORKEY, frame[srcX, srcY])
    end
  end

  # Set actual frame to defined value
  def switchFrame(targetFrame)
    if targetFrame.abs >= @frames.size then
      raise IndexError, "Sprite frame ERROR: #{targetFrame} >= size (#{@frames.size})"
    end

    @displayFrame = targetFrame
  end

  # Switch to next frame
  def nextFrame()
    @displayFrame = (@displayFrame + 1) % @frames.size
  end

  def render(surface, x, y)
    surface.put(@frames[@displayFrame], x, y)
  end
end

class ARESpriteVec < ARESprite
  attr_accessor :position

  def initialize(parentSurface, hwSurface = false)
    super(parentSurface, hwSurface)
    @position = AREVector.new
  end

  def render(surface)
    super(surface, @position.x, @position.y)
  end
end