require 'aresprite.rb'
require 'arevector.rb'

MARS_CENTER = AREVector.new(570, 300)
ENEMIES_COUNT = 3
ACC_CAP = 0.5

class Enemy < ARESpriteVec
  attr_reader :mass, :velocity, :out
  attr_accessor :hp, :havebomb

  def setEnemy(mass, vel, hp, lsrs, dmg)
    @hp = hp
    @mass = mass
    @velocity = vel
    @out = false
    @lasers = lsrs
    @lasercl = rand(4)
    @damage = dmg
    @range = 100 + rand(70)
    @target = nil
    @havebomb = true
  end

  def move()
    radius = MARS_CENTER - @position
    hyp = Math.sqrt(radius.sumsq)
    acc = @mass / radius.sumsq
    tmp = radius / hyp * acc
    tmp.x = ACC_CAP * (tmp.x / tmp.x.abs) if tmp.x.abs > ACC_CAP
    tmp.y = ACC_CAP * (tmp.y / tmp.y.abs) if tmp.y.abs > ACC_CAP
    @velocity += tmp
    @position += @velocity

    if @position.x < 30 or @position.x >= 690 or @position.y < 50 or @position.y > 430
      @out = true
    end

    if @hp <= 0 then
      @out = true
    end
  end

  def fire(map, screen)

    laser_colors = [
      screen.format.mapRGB(0xff, 0x41, 0x1b),
      screen.format.mapRGB(0x23, 0xff, 0xfb),
      screen.format.mapRGB(0x23, 0xff, 0x23),
      screen.format.mapRGB(0xff, 0x29, 0xd1)
    ]

    inrange = Array.new
    MAP_S.times do |i|
      MAP_S.times do |j|
        if not map[i][j].nil?
          s = map[i][j]
          spos = AREVector.new(460 + i * TILE_W + 18, 190 + j * TILE_H + 18)
          dist = Math.sqrt((@position - spos).sumsq)
          inrange << [s, spos] if (dist < @range)
        end
      end
    end

    @target = inrange[rand(inrange.size)] if not inrange.empty? and @target.nil?

    if not @target.nil?
      # fire!
      pos = @target[1]
      s = @target[0]
      screen.lock
      screen.drawLineAlpha(@position.x + 8, @position.y + 8, pos.x, pos.y, laser_colors[@lasercl], 200)
      screen.unlock
      s.hp -= @damage
      # is target still valid?
      dist = Math.sqrt((pos - @position).sumsq)
      @target = nil if dist > @range or s.destroyed
    end
  end
end